﻿using System.Diagnostics;

public class Program {
	private static volatile int s_slowJobsRunning = 0;
	private static volatile int s_slowJobsCompleted = 0;
	private static volatile int s_fastJobsRunning = 0;
	private static volatile int s_fastJobsCompleted = 0;

	private static void Main(string[] args) {
		var sw = Stopwatch.StartNew();

		for (int i = 0; i < 100; i++) {
			ThreadPool.QueueUserWorkItem(_ => {
				Interlocked.Increment(ref s_slowJobsRunning);

				TaskCompletionSource tcs = new TaskCompletionSource();
				ThreadPool.QueueUserWorkItem((object? _) => {
					Interlocked.Increment(ref s_fastJobsRunning);
					Thread.Sleep(50);
					tcs.SetResult();
					Interlocked.Increment(ref s_fastJobsCompleted);
					Interlocked.Decrement(ref s_fastJobsRunning);
				}, null, preferLocal: false);

				tcs.Task.Wait();

				Interlocked.Increment(ref s_slowJobsCompleted);
				Interlocked.Decrement(ref s_slowJobsRunning);
			});
		}

		while (true) {
            Console.WriteLine($"Main: ThreadPool [{ThreadPool.ThreadCount} threads, {ThreadPool.PendingWorkItemCount} pending + {ThreadPool.CompletedWorkItemCount} completed WI] | Jobs (running+completed): {s_slowJobsRunning}+{s_slowJobsCompleted} slow / {s_fastJobsRunning}+{s_slowJobsCompleted} fast");
			Thread.Sleep(100);

			if (ThreadPool.PendingWorkItemCount == 0 && s_slowJobsRunning == 0 && s_fastJobsRunning == 0) {
				break;
			}
        }

		sw.Stop();
        Console.WriteLine($"Total running time: {sw.Elapsed}");
    }
}
