﻿
namespace ContinuationChain {
	internal class Program {

		static void RunTickingSequence() {
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (1) One");
			Thread.Sleep(500);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (2) Two");
			Thread.Sleep(500);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (3) Three");
		}

		static Task RunTickingSequenceAsync() {
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (1) One");
			
			var tcs = new TaskCompletionSource();
			var task1 = Task.Delay(500);
			task1.ContinueWith(_ => {
				Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (2) Two");
				var task2 = Task.Delay(500);
				task2.ContinueWith(_ => {
					Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingSequence: (3) Three");
					tcs.SetResult();
				});
			});
			return tcs.Task;
		}

		static void Main(string[] args) {
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling RunTickingSequence() ...");
			RunTickingSequence();
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: RunTickingSequence() returned ...");

			Console.WriteLine();

			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling RunTickingSequenceAsync() ...");
			var task = RunTickingSequenceAsync();
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: RunTickingSequenceAsync() returned ...");
			do {
				Thread.Sleep(200);
				Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: task.IsCompleted == {task.IsCompleted} ...");
			} while (!task.IsCompleted);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: all done ...");
		}
	}
}