﻿
public class Program {

	public static IEnumerator<Task> GetTickingCoroutine() {
        Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (1) One");
		yield return Task.Delay(500);
        Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (2) Two");
		yield return Task.Delay(500);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (3) Three");
    }

	public static void RunCoroutine(IEnumerator<Task> coroutine) {
		while (coroutine.MoveNext()) {
			coroutine.Current.Wait();
		}
	}

	public static void RunCoroutineInThreads(IEnumerator<Task> coroutine) {
		var t1 = new Thread(() => {
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> custom thread A: calling MoveNext() ...");
			coroutine.MoveNext();
		});
		t1.Start();
		var t2 = new Thread(() => {
			Thread.Sleep(200);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> custom thread B: calling MoveNext() ...");
			coroutine.MoveNext();
		});
		t2.Start();
		var t3 = new Thread(() => {
			Thread.Sleep(400);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> custom thread C: calling MoveNext() ...");
			coroutine.MoveNext();
		});
		t3.Start();
		t1.Join(); t2.Join(); t3.Join();
	}

	public static Task RunCoroutineAsync(IEnumerator<Task> coroutine) {
		if (!coroutine.MoveNext()) {
			return Task.CompletedTask;
		}

		var tcs = new TaskCompletionSource();
		coroutine.Current.ContinueWith(executeOneStep);
		return tcs.Task;

		void executeOneStep(Task prevTask) {
			if (coroutine.MoveNext()) {
				coroutine.Current.ContinueWith(executeOneStep);
			} else {
				tcs.SetResult();
			}
		}
	}

	private static void Main(string[] args) {
        Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling GetTickingCoroutine() ...");
		var c1 = GetTickingCoroutine();
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling RunCoroutine() ...");
		RunCoroutine(c1);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: RunCoroutine() returned ...");

		Console.WriteLine();
		Console.WriteLine("Press ENTER to start example 2 ...");
		Console.ReadLine();

		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling GetTickingCoroutine() ...");
		var c2 = GetTickingCoroutine();
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling RunCoroutine() ...");
		RunCoroutineInThreads(c2);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: RunCoroutineInThreads() returned ...");

		Console.WriteLine();
		Console.WriteLine("Press ENTER to start example 3 ...");
		Console.ReadLine();

		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling GetTickingCoroutine() ...");
		var c3 = GetTickingCoroutine();
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling RunCoroutineAsync() ...");
		var task3 = RunCoroutineAsync(c3);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: RunCoroutineAsync() returned ...");
		do {
			Thread.Sleep(200);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: task3.IsCompleted == {task3.IsCompleted} ...");
		} while (!task3.IsCompleted);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: all done ...");

		Console.WriteLine();
		Console.WriteLine("Press ENTER to start example 4 ...");
		Console.ReadLine();

		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: calling TickingMethodAsync() ...");
		var task4 = TickingMethodAsync();
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: TickingMethodAsync() returned ...");
		do {
			Thread.Sleep(200);
			Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: task4.IsCompleted == {task4.IsCompleted} ...");
		} while (!task4.IsCompleted);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> Main: all done ...");
	}

	private static IEnumerator<Task> InternalGetTickingCoroutine(int someParamA, int someParamB) {
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (1) One");
		yield return Task.Delay(500);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (2) Two");
		yield return Task.Delay(500);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingCoroutine: (3) Three");
	}

	public static Task TickingMethodAsync_this_is_so_much_work(int someParamA, int someParamB) {
		var coroutine = InternalGetTickingCoroutine(someParamA, someParamB);
		return RunCoroutineAsync(coroutine);
	}

	public static async Task TickingMethodAsync() {
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingMethodAsync: (1) One");
		await Task.Delay(500);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingMethodAsync: (2) Two");
		await Task.Delay(500);
		Console.WriteLine($"Thread {Thread.CurrentThread.ManagedThreadId} >> TickingMethodAsync: (3) Three");
	}
}
