﻿
public class Program {

	public static void RunAll(Task[] tasks) {
		int sum = 0;
		foreach (Task task in tasks) {
            Console.WriteLine($"RunAll: .Wait()-ing for {task.GetType()} ...");
            try {
				if (task is Task<int> futureInt) {
					sum += futureInt.Result;
                } else {
					task.Wait();
				}
			} catch (Exception ex) {
				Console.WriteLine($"RunAll: caught exception {ex.GetType()}:");
                Console.WriteLine($"\t.InnerException == {ex.InnerException?.GetType()}");
				if (ex is AggregateException aggregateEx) {
					Console.WriteLine($"\t.InnerExceptions (.Count == {aggregateEx.InnerExceptions.Count}):");
					foreach (var innerEx in aggregateEx.InnerExceptions) {
                        Console.WriteLine($"\t\t{innerEx.GetType()}");
					}
				}
            }
		}
		Console.WriteLine($"RunAll: DONE (sum == {sum}) ...");
	}

	public static void Main(string[] args) {
		var cts = new CancellationTokenSource();
		cts.Cancel();

		var t1 = Task.FromException<int>(new IndexOutOfRangeException());
		var t2 = Task.FromCanceled<int>(cts.Token);
		var t3 = Task.WhenAll(
			Task.FromException<int>(new OutOfMemoryException()),
			t1, t2,
			Task.FromException<int>(new NullReferenceException())
		);
		var tasks = new Task[] { t1, t2, t3 };

		RunAll(tasks);
		Console.WriteLine();
		Console.WriteLine("Press ENTER to start async method with await...");
		Console.ReadLine();
		RunAllAsync(tasks).Wait();
    }

	public static async Task RunAllAsync(Task[] tasks) {
		int sum = 0;
		foreach (Task task in tasks) {
			Console.WriteLine($"RunAllAsync: awaiting for {task.GetType()} ...");
			try {
				if (task is Task<int> futureInt) {
					sum += await futureInt;
				} else {
					await task;
				}
			} catch (Exception ex) {
				Console.WriteLine($"RunAllAsync: caught exception {ex.GetType()}:");
				Console.WriteLine($"\t.InnerException == {ex.InnerException?.GetType()}");
				if (ex is AggregateException aggregateEx) {
					Console.WriteLine($"\t.InnerExceptions (.Count == {aggregateEx.InnerExceptions.Count}):");
					foreach (var innerEx in aggregateEx.InnerExceptions) {
						Console.WriteLine($"\t\t{innerEx.GetType()}");
					}
				}
			}
		}
		Console.WriteLine($"RunAllAsync: DONE (sum == {sum}) ...");
	}
}